/*********************************************************************
The program box places a box around a text. It takes the text from
the standard input or a file and prints the boxed text on the standard
output. TAB's are converted to spaces. Distance between tabstops is
considered to be 8 characters. 
Terminal width is 80 by default. This can be changed, EG by "box -w 132".
The flag "-c" causes box to center the box.
*********************************************************************/

#include <stdio.h>

#define TRUE 1
#define FALSE 0
#define TAB_STOP 8

main(argc,argv)
int argc;
char **argv;
{
    register char c;
    register int i, j, length, tab_l, argp, center=FALSE, width = 80, indent;
    register int tmp_flag;
    FILE *infile, *tmpfile;
    char path[30], *name;

    for ( argp=1; argp<argc; argp++ ) {	/* Get command line flags */
	if (*argv[argp] != '-')
	    break;
	for ( j=1; argv[argp][j] != '\0'; j++ )
	    switch ( argv[argp][j] ) {
		case 'c':
		    center = TRUE;
		    break;
		case 'w':
		    if (++argp < argc)
			width = atoi(argv[argp]);
		    else
			goto usage;
		    break;
		default:
usage:		    fprintf(stderr,
			"Usage: box [-c] [-w width] [file]\n");
		    exit(1);
		    }
	}

    if (tmp_flag = (argc == argp)) {	/* Standard input */
	infile = stdin;

	/* Create a temporary file: */
	strcpy(path, "/usr/tmp/box_tmpXXXXXX");
	name = path+9;
	mktemp(name);
	if ((tmpfile = fopen(path, "w")) == NULL) {
	    fprintf(stderr, "box: can't create temporary file");
	    exit(1);
	    }
	}
    else if (argc > argp+1)
	fprintf(stderr, "box: Too many arguments\n");
    else if ((infile = fopen(argv[argp], "r")) == NULL) {
	fprintf(stderr, "box: can't open file %s\n", argv[argp]);
	exit(1);
	}

    /* Pass 1: Look for the longest line */
    length = i = 0;
    while ((c=getc(infile)) != EOF) {
	if (tmp_flag)
	    putc(c, tmpfile);
	if (c == '\n') {
newline:    if (i > length)
		length = i;
	    i = 0;
	    }
	else if (c == '\t') {	/* TAB's will be converted to spaces */
	    i += TAB_STOP - i%TAB_STOP;
	    if (i >= width-4) {
		i = width-4;
		goto newline;
		}
	    }
	else {
	    i++;
	    if (i >= width-4)
		goto newline;
	    }
	}
    if (i)	/* File did not end with a '\n' */
	goto newline;

    /* Open file for pass 2: */
    if (tmp_flag) {
	fclose(tmpfile);
	fclose(infile);
	if ((infile = fopen(path, "r")) == NULL) {
	    fprintf(stderr, "box: Can't open temporary file\n");
	    exit(1);
	    }
	unlink(path);
	}
    else {
	fclose(infile);
	if ((infile = fopen(argv[argp], "r")) == NULL) {
	    fprintf(stderr, "box: can't open file %s\n", argv[argp]);
	    exit(1);
	    }
	}

    /* Indent if text is to be centered: */
    indent = center ? (width-(length+4))/2 : 0;

    for (i=0; i<indent; i++)
	putchar(' ');
    putchar('+');
    for (i=0; i<length+2; i++)
	putchar('-');
    printf("+\n");
    i = 0;
    while ((c=getc(infile)) != EOF) {
	if (i == 0) {
	    for (j=0; j<indent; j++)
		putchar(' ');
	    printf("! ");
	    }
	if (c == '\n') {
newl:	    for (; i<length; i++)
		putchar(' ');
	    i = 0;
	    printf(" !\n");
	    }
	else if (c == '\t') {
	    tab_l = TAB_STOP- i%TAB_STOP;
	    for (j=0; j < tab_l; j++) {
		putchar(' ');
		i++;
		if (i >= width-4)
		    goto newl;
		}
	    }
	else {
	    putchar(c);
	    i++;
	    if (i >= width-4)
		goto newl;
	    }
	}
    if (i)	/* File did not end with a '\n' */
	goto newl;
    for (i=0; i<indent; i++)
	putchar(' ');
    putchar('+');
    for (i=0; i<length+2; i++)
	putchar('-');
    printf("+\n");
}
