#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/termio.h>
#include <signal.h>

#define STDIN 0
#define ON 1
#define OFF 0
#define PORTRAIT 0
#define LANDSCAPE 1
#define VSX 0
#define VSY 1
#define FSX 2
#define FSY 3

main(argc, argv)
int argc;
char *argv[];
{
    int *catch();
    int i, pk, mode, input;
    int win[4];
    char c, buf[8], *termname, *ttyname(), *strrchr();

    if ((input = open("/dev/tty", O_RDONLY)) == -1)
	exit(1);
    dup2(input, STDIN);
    sterm(ON);
    signal(SIGALRM, catch);

    /* Check if it is a pk terminal */
    termname = ttyname(STDIN);
    pk = (strncmp(termname, "/dev/pk", 7) == 0);

    /* Check if vt100 compatible */
    fprintf(stderr, "\033[6n");
    alarm(2);
    if (read(STDIN, &c, 1) != 1) {
	printf("adm3a");
	goto end;
	}
    while (c != 'R')
	read(STDIN, &c, 1);

    /* Check if facit twist compatible */
    fprintf(stderr, "\033[?1n");
    alarm(2);
    if (read(STDIN, &c, 1) != 1) {
	printf("vt100");
	goto end;
	}
    for (i=0; i<3; i++)
	read(STDIN, &c, 1);
    /* Portrait or landscape? */
    if (c == 'P')
	mode = PORTRAIT;
    else
	mode = LANDSCAPE;
    read(STDIN, &c, 1);

    /* Check if ABC1600 */
    fprintf(stderr, "\033:3n");
    alarm(2);
    if (read(STDIN, &c, 1) != 1) {
	if (mode == PORTRAIT)
	    printf("twi72");
	else
	    printf("twist");
	goto end;
	}

    /* This was an ABC1600 */
    /* Get data about window size etc */
    read(STDIN, &c, 1);
    for (i=0; i<4; i++) {
	win[i] = 0;
	read(STDIN, &c, 1);
	while (c != ';') {
	    win[i] = win[i]*10 + c-'0';
	    read(STDIN, &c, 1);
	    }
	}
    while (c != 'W')
	read(STDIN, &c, 1);

    if (argc == 1) /* Terminal type */
	if (mode == PORTRAIT)
	    if (win[VSX]==768 && win[VSY]==1024)
		printf("abc1600p");
	    else
		printf(pk ? "pkwin" : "win");
	else
	    if (win[VSX]==1024 && win[VSY]==768)
		printf("abc1600l");
	    else
		printf(pk ? "pkwin" : "win");
    else { /* TERMCAP */
	printf(pk ? "wp|pkwin|pk window" : "w0|win|w");
	printf(":co#%d:li#%d:tc=abc1600w:", 
		win[VSX]/win[FSX], win[VSY]/win[FSY]);
	}
end:
    sterm(OFF);
    exit(0);
}

sterm(mode)
int mode;
{
    struct termio tty;

    ioctl(STDIN, TCGETA, &tty);
    if (mode == ON)
	tty.c_lflag &= ~(ICANON | ECHO);
    else
	tty.c_lflag |= (ICANON | ECHO);
    ioctl(STDIN, TCSETA, &tty);
}

int *catch()
{
    signal(SIGALRM, catch);
}
